/// <reference types="node" />
import { IFieldProcessor, IFieldProcessorEvents } from "./field-processor";
import { BinaryStreamMarker, BinaryStream, StringStream } from "./lazy-ref";
export declare namespace RecorderDecorators {
    /**
     * L: In case of LazyRef this is first type parameter of LazyRef.
     */
    interface PropertyOptions<L> {
        persistent: boolean;
        lazyDirectRawRead?: boolean;
        lazyDirectRawWrite?: boolean;
        fieldProcessorResolver?: () => IFieldProcessor<L>;
        /** Framework internal use. */
        fieldProcessorEvents?: IFieldProcessorEvents<L>;
    }
    /**
     * Decorator for get property.
     * \@RecorderDecorators.property() is equivalent to \@RecorderDecorators.property({persistent: true})
     *
     * Examplo:
     * ```ts
       ...
       private _myField: string;
       @RecorderDecorators.property()
       public get myField(): string {
         return this._myField;
       }
       public set myField(value: string) {
         this._myField = value;
       }
       ...
     * ```
     */
    function property<T>(options: PropertyOptions<T>): MethodDecorator;
    function property<T>(): MethodDecorator;
    function playerObjectId<T>(): MethodDecorator;
    /**
     * Used with {@link RecorderDecorators#playerType}.
     */
    interface playerTypeOptions {
        /**
         * Mapped player side class type.
         */
        playerType: string;
        /**
         * Use it if you have more than one typescript classes mapping the same player side class type.
         */
        disambiguationId?: string;
    }
    /**
     * Decorator for persistent entity.
     *
     * Sample:
     * ```ts
     * ...
     * @JsonPlayback.playerType({playerType: 'org.mypackage.MyPersistentEntity'})
     * export class MyPersistentEntityJs {
     * ...
     * ```
     */
    function playerType<T>(options: playerTypeOptions): ClassDecorator;
    const DateProcessor: IFieldProcessor<Date>;
    const BufferProcessor: IFieldProcessor<Buffer>;
    const StringProcessor: IFieldProcessor<String>;
    const BinaryStreamProcessor: IFieldProcessor<BinaryStream>;
    const StringStreamProcessor: IFieldProcessor<StringStream>;
    const TypeProcessorEntries: ({
        type: {
            new (str: string, encoding?: string): Buffer;
            new (size: number): Buffer;
            new (array: Uint8Array): Buffer;
            new (arrayBuffer: ArrayBuffer | SharedArrayBuffer): Buffer;
            new (array: any[]): Buffer;
            new (buffer: Buffer): Buffer;
            prototype: Buffer;
            from(arrayBuffer: ArrayBuffer | SharedArrayBuffer, byteOffset?: number, length?: number): Buffer;
            from(data: any[]): Buffer;
            from(data: Uint8Array): Buffer;
            from(str: string, encoding?: string): Buffer;
            of(...items: number[]): Buffer;
            isBuffer(obj: any): obj is Buffer;
            isEncoding(encoding: string): boolean;
            byteLength(string: string | ArrayBuffer | SharedArrayBuffer | Uint8Array | Uint8ClampedArray | Uint16Array | Uint32Array | Int8Array | Int16Array | Int32Array | Float32Array | Float64Array | DataView, encoding?: string): number;
            concat(list: Uint8Array[], totalLength?: number): Buffer;
            compare(buf1: Uint8Array, buf2: Uint8Array): number;
            alloc(size: number, fill?: string | number | Buffer, encoding?: string): Buffer;
            allocUnsafe(size: number): Buffer;
            allocUnsafeSlow(size: number): Buffer;
            poolSize: number;
        };
        processor: IFieldProcessor<Buffer>;
    } | {
        type: StringConstructor;
        processor: IFieldProcessor<String>;
    } | {
        type: typeof BinaryStreamMarker;
        processor: IFieldProcessor<BinaryStream>;
    } | {
        type: DateConstructor;
        processor: IFieldProcessor<Date>;
    })[];
}
